const { getParamsAndQuery } = require('../utils')

const METHODS = ['GET', 'POST', 'PUT', 'PATCH', 'DELETE', 'HEAD', 'OPTIONS']

let found = false

const methodFn = method => (path, handler) => {
  if (!path) throw new Error('You need to set a valid path')
  if (!handler) throw new Error('You need to set a valid handler')

  return (req, res) => {
    const { params, query } = getParamsAndQuery(path, req.url)

    if (!found && params && req.method === method) {
      found = true
      return handler(Object.assign(req, { params, query }), res)
    }
  }
}

exports.router = (...funcs) => async (req, res) => {
  found = false
  funcs.map(fn => fn(req, res)).find(fn => fn)
}

METHODS.forEach(method => {
  exports[method.toLowerCase()] = methodFn(method)
})
